package xyz.lychee.lagfixer.libs.adventure.text.renderer;

import xyz.lychee.lagfixer.libs.adventure.text.BlockNBTComponent;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.EntityNBTComponent;
import xyz.lychee.lagfixer.libs.adventure.text.KeybindComponent;
import xyz.lychee.lagfixer.libs.adventure.text.NBTComponent;
import xyz.lychee.lagfixer.libs.adventure.text.ScoreComponent;
import xyz.lychee.lagfixer.libs.adventure.text.SelectorComponent;
import xyz.lychee.lagfixer.libs.adventure.text.StorageNBTComponent;
import xyz.lychee.lagfixer.libs.adventure.text.TextComponent;
import xyz.lychee.lagfixer.libs.adventure.text.TranslatableComponent;
import xyz.lychee.lagfixer.libs.annotations.NotNull;

public abstract class AbstractComponentRenderer<C> implements ComponentRenderer<C> {
   public @NotNull Component render(final @NotNull Component component, final @NotNull C context) {
      if (component instanceof TextComponent) {
         return this.renderText((TextComponent)component, context);
      } else if (component instanceof TranslatableComponent) {
         return this.renderTranslatable((TranslatableComponent)component, context);
      } else if (component instanceof KeybindComponent) {
         return this.renderKeybind((KeybindComponent)component, context);
      } else if (component instanceof ScoreComponent) {
         return this.renderScore((ScoreComponent)component, context);
      } else if (component instanceof SelectorComponent) {
         return this.renderSelector((SelectorComponent)component, context);
      } else {
         if (component instanceof NBTComponent) {
            if (component instanceof BlockNBTComponent) {
               return this.renderBlockNbt((BlockNBTComponent)component, context);
            }

            if (component instanceof EntityNBTComponent) {
               return this.renderEntityNbt((EntityNBTComponent)component, context);
            }

            if (component instanceof StorageNBTComponent) {
               return this.renderStorageNbt((StorageNBTComponent)component, context);
            }
         }

         return component;
      }
   }

   protected abstract @NotNull Component renderBlockNbt(final @NotNull BlockNBTComponent component, final @NotNull C context);

   protected abstract @NotNull Component renderEntityNbt(final @NotNull EntityNBTComponent component, final @NotNull C context);

   protected abstract @NotNull Component renderStorageNbt(final @NotNull StorageNBTComponent component, final @NotNull C context);

   protected abstract @NotNull Component renderKeybind(final @NotNull KeybindComponent component, final @NotNull C context);

   protected abstract @NotNull Component renderScore(final @NotNull ScoreComponent component, final @NotNull C context);

   protected abstract @NotNull Component renderSelector(final @NotNull SelectorComponent component, final @NotNull C context);

   protected abstract @NotNull Component renderText(final @NotNull TextComponent component, final @NotNull C context);

   protected abstract @NotNull Component renderTranslatable(final @NotNull TranslatableComponent component, final @NotNull C context);
}
